@@ -44,11 +44,14 @@ try:
 except:
     from mercurial.httprepo import passwordmgr
 from mercurial import mail
+from mercurial import encoding
 from urllib2 import AbstractBasicAuthHandler, AbstractDigestAuthHandler
 
-# mercurial.demandimport incompatibility workaround, various keyring
-# backends fail due to it. This should not be necessary once next
-# workaround is present, but to be safe I preserve this setting.
+# mercurial.demandimport incompatibility workaround.
+# various keyring backends fail as they can't properly import helper
+# modules (as demandimport modifies python import behaviour).
+# If you get import errors with demandimport in backtrace, try
+# guessing what to block and extending the list below.
 from mercurial import demandimport
 for blocked_module in [
     "gobject._gobject",
@@ -66,13 +69,14 @@ for blocked_module in [
     "atom.token_store",
     "ctypes",
     "secretstorage.exceptions",
+    "fs.opener",
     ]:
     if blocked_module not in demandimport.ignore:
         demandimport.ignore.append(blocked_module)
 
 # Temporarily disable demandimport to make the need of extending
-# list above less likely.
-if __import__ == demandimport._import:
+# the list above less likely.
+if demandimport.isenabled():
     demandimport.disable()
     try:
         import keyring
@@ -110,29 +114,39 @@ class PasswordStore(object):
     def __init__(self):
         self.cache = dict()
     def get_http_password(self, url, username):
-        return keyring.get_password(KEYRING_SERVICE,
-                                    self._format_http_key(url, username))
+        return self._read_password_from_keyring(
+            self._format_http_key(url, username))
     def set_http_password(self, url, username, password):
-        keyring.set_password(KEYRING_SERVICE,
-                             self._format_http_key(url, username),
-                             password)
+        self._save_password_to_keyring(
+            self._format_http_key(url, username),
+            password)
     def clear_http_password(self, url, username):
         self.set_http_password(url, username, "")
     def _format_http_key(self, url, username):
         return "%s@@%s" % (username, url)
     def get_smtp_password(self, machine, port, username):
-        return keyring.get_password(
-            KEYRING_SERVICE,
+        return self._read_password_from_keyring(
             self._format_smtp_key(machine, port, username))
     def set_smtp_password(self, machine, port, username, password):
-        keyring.set_password(
-            KEYRING_SERVICE,
+        self._save_password_to_keyring(
             self._format_smtp_key(machine, port, username),
             password)
     def clear_smtp_password(self, machine, port, username):
         self.set_smtp_password(machine, port, username, "")
     def _format_smtp_key(self, machine, port, username):
         return "%s@@%s:%s" % (username, machine, str(port))
+    def _read_password_from_keyring(self, pwdkey):
+        password = keyring.get_password(KEYRING_SERVICE, pwdkey)
+        # Reverse recoding from next routine
+        if isinstance(password, unicode):
+            return encoding.tolocal(password.encode('utf-8'))
+        return password
+    def _save_password_to_keyring(self, pwdkey, password):
+        # keyring in general expects unicode. Mercurial provides "local" encoding. See #33
+        password = encoding.fromlocal(password).decode('utf-8')
+        keyring.set_password(KEYRING_SERVICE,
+                             pwdkey, 
+                             password)
 
 password_store = PasswordStore()
 
